package net.minecraft.src;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// SDK ModLoader
@SuppressWarnings("rawtypes")
public class ModLoaderMp {
    public static final String NAME = "ModLoaderMP";

    private static boolean hasInit = false;
	private static Map<Class, Pair<Integer, Integer>> entityTrackerMap = null;
	private static Map<Class, Integer> entityTrackerEntryMap = null;
	private static List<String> bannedMods = new ArrayList<String>();
    private static Method method_R;
    private static Field field_bH;
    
    public static void Init()
    {
		if(!hasInit)
            init();
    }
    
    public static void AddAllEntityTrackerEntries()
    {
		if(!hasInit)
            init();
		
    	entityTrackerEntryMap = new HashMap<Class, Integer>();
    	for(int m = 0; m < ModLoader.getLoadedMods().size(); m++)
        {
    		BaseMod baseMod = (BaseMod)ModLoader.getLoadedMods().get(m);
    		
    		if(baseMod instanceof BaseModMp)
    			((BaseModMp)baseMod).AddEntityTrackerEntries(entityTrackerEntryMap);
        }
    }
	
	public static void HandleAllLogins(EntityPlayerMP entityPlayerMp)
	{
		if(!hasInit)
            init();
		
		sendModCheck(entityPlayerMp);
		for(int m = 0; m < ModLoader.getLoadedMods().size(); m++)
        {
			BaseMod baseMod = (BaseMod)ModLoader.getLoadedMods().get(m);
			
			if(baseMod instanceof BaseModMp)
				((BaseModMp)baseMod).HandleLogin(entityPlayerMp);
        }
	}

	public static void HandleAllPackets(Packet200ModLoader packet200ModLoader, EntityPlayerMP entityPlayerMp)
	{
		if(!hasInit)
            init();
		
		if(packet200ModLoader.modId == NAME.hashCode())
			handleModCheckResponse(packet200ModLoader, entityPlayerMp);
		else
			for(int m = 0; m < ModLoader.getLoadedMods().size(); m++)
	        {
				BaseMod baseMod = (BaseMod)ModLoader.getLoadedMods().get(m);
				
				if(baseMod instanceof BaseModMp)
				{
					BaseModMp baseModMultiplayer = (BaseModMp)baseMod;
					
		            if (baseModMultiplayer.getId() == packet200ModLoader.modId)
		            {
		            	baseModMultiplayer.HandlePacket(packet200ModLoader, entityPlayerMp);
		            	break;
		            }
				}
	        }
	}
    
    public static void OpenModGUI(EntityPlayer entityplayer, IInventory iinventory, CraftingInventoryCB craftinginventorycb)
    {
		if(!hasInit)
            init();
        
        if(entityplayer instanceof EntityPlayerMP)
        {
        	EntityPlayerMP entityplayermp = (EntityPlayerMP)entityplayer;
        	
        	try
        	{
	        	method_R.invoke(entityplayermp);
	        	int bH = field_bH.getInt(entityplayermp);
		    	entityplayermp.playerNetServerHandler.sendPacket(new Packet100(bH, 2, iinventory.getInvName(), iinventory.getSizeInventory()));
		    	entityplayermp.craftingInventory = craftinginventorycb;
		    	entityplayermp.craftingInventory.windowId = bH;
		    	entityplayermp.craftingInventory.func_20128_a(entityplayermp);
        	}
        	catch (InvocationTargetException invocationtargetexception)
        	{
        		ModLoader.getLogger().throwing("ModLoaderMultiplayer", "OpenModGUI", invocationtargetexception);
        		ModLoader.ThrowException("ModLoaderMultiplayer", invocationtargetexception);
        	}
        	catch (IllegalAccessException illegalaccessexception)
        	{
        		ModLoader.getLogger().throwing("ModLoaderMultiplayer", "OpenModGUI", illegalaccessexception);
        		ModLoader.ThrowException("ModLoaderMultiplayer", illegalaccessexception);
        	}
        }
    }

	public static void HandleEntityTrackers(EntityTracker entityTracker, Entity entity)
	{
		if(!hasInit)
            init();
		
		Iterator<Map.Entry<Class, Pair<Integer, Integer>>> it = entityTrackerMap.entrySet().iterator();
    	while (it.hasNext())
    	{
    	    Map.Entry<Class, Pair<Integer, Integer>> entry = it.next();
    	    
    	    if(entry.getKey().isInstance(entity))
    	    {
    	    	entityTracker.trackEntity(entity, entry.getValue().getLeft(), entry.getValue().getRight(), true);
    	    	return;
    	    }
    	}
	}

	public static int HandleEntityTrackerEntries(Entity entity)
	{
		if(!hasInit)
            init();
		
		if(entityTrackerEntryMap == null)
		{
			AddAllEntityTrackerEntries();
		}
        
		if(entityTrackerEntryMap.containsKey(entity.getClass()))
		{
			return entityTrackerEntryMap.get(entity.getClass());
		}
		else
		{
			return -1;
		}
	}
	
	public static void SendPacketToAll(BaseModMp baseModMultiplayer, Packet200ModLoader packet200ModLoader)
	{
		if(!hasInit)
            init();
		
		if(baseModMultiplayer == null)
			throw new IllegalArgumentException("baseModMultiplayer cannot be null.");
		
		packet200ModLoader.modId = baseModMultiplayer.getId();
		for(int i = 0; i < ModLoader.getMinecraftServerInstance().worldMngr.playerEntities.size(); i++)
			((EntityPlayerMP)ModLoader.getMinecraftServerInstance().worldMngr.playerEntities.get(i)).playerNetServerHandler.sendPacket(packet200ModLoader);
	}
	
	public static void SendPacketTo(BaseModMp baseModMultiplayer, EntityPlayerMP entityPlayer, Packet200ModLoader packet200ModLoader)
	{
		if(!hasInit)
            init();
		
		if(baseModMultiplayer == null)
			throw new IllegalArgumentException("baseModMultiplayer cannot be null.");
		
		packet200ModLoader.modId = baseModMultiplayer.getId();
		SendPacketTo(entityPlayer, packet200ModLoader);
	}
    
    private static void init()
    {
        hasInit = true;

        try
        {
        	// Add custom packet mapping
        	Method method_AddIdClassMapping;
            try
            {
                method_AddIdClassMapping = (Packet.class).getDeclaredMethod("a", new Class[] {
                		int.class, Class.class
                });
            }
            catch(NoSuchMethodException nosuchmethodexception)
            {
            	method_AddIdClassMapping = (Packet.class).getDeclaredMethod("addIdClassMapping", new Class[] {
            			int.class, Class.class
                });
            }
            method_AddIdClassMapping.setAccessible(true);
            method_AddIdClassMapping.invoke(null, 200, Packet200ModLoader.class);
            
            // Get method R in EntityPlayer
            try
            {
            	method_R = (EntityPlayerMP.class).getDeclaredMethod("V", (Class[])null);
            }
            catch(NoSuchMethodException nosuchmethodexception)
            {
            	method_R = (EntityPlayerMP.class).getDeclaredMethod("func_20060_R", (Class[])null);
            }
            method_R.setAccessible(true);
            
            // Get field bH in EntityPlayerMP
            try
            {
            	field_bH = (EntityPlayerMP.class).getDeclaredField("bH");
            }
            catch(NoSuchFieldException nosuchfieldexception)
            {
            	field_bH = (EntityPlayerMP.class).getDeclaredField("field_20065_bH");
            }
            field_bH.setAccessible(true);
        }
		catch (IllegalAccessException illegalaccessexception)
		{
			ModLoader.getLogger().throwing("ModLoader", "AddCustomPacketMapping", illegalaccessexception);
			ModLoader.ThrowException("ModLoaderMultiplayer", illegalaccessexception);
		}
        catch(IllegalArgumentException illegalargumentexception)
        {
        	ModLoader.getLogger().throwing("ModLoader", "init", illegalargumentexception);
        	ModLoader.ThrowException("ModLoaderMultiplayer", illegalargumentexception);
            throw new RuntimeException(illegalargumentexception);
        }
		catch(InvocationTargetException invocationtargetexception)
		{
			ModLoader.getLogger().throwing("ModLoader", "init", invocationtargetexception);
			ModLoader.ThrowException("ModLoaderMultiplayer", invocationtargetexception);
		}
        catch(NoSuchFieldException nosuchfieldexception)
        {
        	ModLoader.getLogger().throwing("ModLoader", "init", nosuchfieldexception);
        	ModLoader.ThrowException("ModLoaderMultiplayer", nosuchfieldexception);
            throw new RuntimeException(nosuchfieldexception);
        }
        catch(NoSuchMethodException nosuchmethodexception2)
        {
        	ModLoader.getLogger().throwing("ModLoader", "init", nosuchmethodexception2);
        	ModLoader.ThrowException("ModLoaderMultiplayer", nosuchmethodexception2);
            throw new RuntimeException(nosuchmethodexception2);
        }
        catch(SecurityException securityexception)
        {
        	ModLoader.getLogger().throwing("ModLoader", "init", securityexception);
        	ModLoader.ThrowException("ModLoaderMultiplayer", securityexception);
            throw new RuntimeException(securityexception);
        }
        
        // Add all entity trackers
        entityTrackerMap = new HashMap<Class, Pair<Integer, Integer>>();
    	for(int i = 0; i < ModLoader.getLoadedMods().size(); i++)
        {
    		BaseMod baseMod = (BaseMod)ModLoader.getLoadedMods().get(i);
    		if(baseMod instanceof BaseModMp)
    			((BaseModMp)baseMod).AddEntityTrackers(entityTrackerMap);
        }
        
    	// Load banned mods
    	try
    	{
    		// Get banned mods file
    		File bannedModFile = ModLoader.getMinecraftServerInstance().getFile("banned-mods.txt");
    		
    		// Create it if it doesn't exist
    		if(!bannedModFile.exists())
    			bannedModFile.createNewFile();
    		
    		// Read banned mods
    		BufferedReader bannedModReader = new BufferedReader(new InputStreamReader(new FileInputStream(bannedModFile)));
    		String line;
    		while ((line = bannedModReader.readLine()) != null)
    			bannedMods.add(line);
    	}
    	catch (FileNotFoundException filenotfoundexception)
    	{
    		ModLoader.getLogger().throwing("ModLoader", "init", filenotfoundexception);
        	ModLoader.ThrowException("ModLoaderMultiplayer", filenotfoundexception);
            throw new RuntimeException(filenotfoundexception);
    	}
    	catch (IOException ioexception)
    	{
    		ModLoader.getLogger().throwing("ModLoader", "init", ioexception);
        	ModLoader.ThrowException("ModLoaderMultiplayer", ioexception);
            throw new RuntimeException(ioexception);
    	}
    }
	
	private static void SendPacketTo(EntityPlayerMP entityPlayer, Packet200ModLoader packet200ModLoader)
	{
		entityPlayer.playerNetServerHandler.sendPacket(packet200ModLoader);
	}

	private static void sendModCheck(EntityPlayerMP entityPlayerMp)
	{
		// Send mod check packet
		Packet200ModLoader packetModCheck = new Packet200ModLoader();
		packetModCheck.modId = NAME.hashCode();
		SendPacketTo(entityPlayerMp, packetModCheck);
	}

	private static void handleModCheckResponse(Packet200ModLoader packet200ModLoader, EntityPlayerMP entityPlayerMp)
	{
		// Check for banned mods
		List<String> modsBanned = new ArrayList<String>();
		for(int i = 0; i < bannedMods.size(); i++)
			for(int j = 0; j < packet200ModLoader.dataString.length; j++)
				if(packet200ModLoader.dataString[j].substring(packet200ModLoader.dataString[j].lastIndexOf("mod_")).startsWith(bannedMods.get(i)))
					modsBanned.add(packet200ModLoader.dataString[j]);
		
		// Check for missing mods
		List<String> modsMissing = new ArrayList<String>();
		for(int i = 0; i < ModLoader.getLoadedMods().size(); i++)
		{
			boolean modFound = false;
			for(int j = 0; j < packet200ModLoader.dataString.length; j++)
			{
				if(((BaseMod)ModLoader.getLoadedMods().get(i)).toString().equals(packet200ModLoader.dataString[j]))
				{
					modFound = true;
					break;
				}
			}
			
			if(!modFound)
				modsMissing.add(packet200ModLoader.dataString[i]);
		}
		
		// Boot for banned or missing mods
		if(modsBanned.size() != 0)
		{
			StringBuilder modsBannedString = new StringBuilder();
			for(int i = 0; i < modsBanned.size(); i++)
			{
				modsBannedString.append("\n");
				modsBannedString.append(modsBanned.get(i).substring(modsBanned.get(i).lastIndexOf("mod_")));
			}
			
			entityPlayerMp.playerNetServerHandler.func_43_c("The following mods are banned on this server:" + modsBannedString.toString());
		}
		else if(modsMissing.size() != 0)
		{
			StringBuilder modsMissingString = new StringBuilder();
			for(int i = 0; i < modsMissing.size(); i++)
			{
				modsMissingString.append("\n");
				modsMissingString.append(modsMissing.get(i).substring(modsMissing.get(i).lastIndexOf("mod_")));
			}
			
			entityPlayerMp.playerNetServerHandler.func_43_c("You are missing the following mods:" + modsMissingString.toString());
		}
	}
}
