package net.minecraft.src;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

// SDK ModLoader
public class Packet200ModLoader extends Packet {
    public Packet200ModLoader() { }
    
    @Override
    public void readPacketData(DataInputStream datainputstream) throws IOException
    {
    	// Read modId
        modId = datainputstream.readInt();
        
        // Read packetType
        packetType = datainputstream.readInt();
        
        // Read dataInt
        int dataIntSize = datainputstream.readInt();
        dataInt = new int[dataIntSize];
        for(int i = 0; i < dataIntSize; i++)
        {
        	dataInt[i] = datainputstream.readInt();
        }
        
        // Read dataFloat
        int dataFloatSize = datainputstream.readInt();
        dataFloat = new float[dataFloatSize];
        for(int i = 0; i < dataFloatSize; i++)
        {
        	dataFloat[i] = datainputstream.readFloat();
        }
        
        // Read dataString
        int dataStringSize = datainputstream.readInt();
        dataString = new String[dataStringSize];
        for(int i = 0; i < dataStringSize; i++)
        {
        	// Get length of string
        	int length = datainputstream.readInt();
        	
        	// Get bytes
        	byte[] dataStringBytes = new byte[length];
        	datainputstream.read(dataStringBytes, 0, length);
        	
        	// Convert to string
        	dataString[i] = new String(dataStringBytes);
        }
    }
    
    @Override
    public void writePacketData(DataOutputStream dataoutputstream) throws IOException
    {
    	// Write modId
    	dataoutputstream.writeInt(modId);
    	
    	// Write packetType
    	dataoutputstream.writeInt(packetType);
    	
    	// Write dataInt
    	dataoutputstream.writeInt(dataInt.length);
    	for(int i = 0; i < dataInt.length; i++)
    	{
    		dataoutputstream.writeInt(dataInt[i]);
    	}
    	
    	// Write dataFloat
    	dataoutputstream.writeInt(dataFloat.length);
    	for(int i = 0; i < dataFloat.length; i++)
    	{
    		dataoutputstream.writeFloat(dataFloat[i]);
    	}
    	
    	// Write dataString
    	dataoutputstream.writeInt(dataString.length);
    	for(int i = 0; i < dataString.length; i++)
    	{
    		// Length for each string, then the string
    		dataoutputstream.writeInt(dataString[i].length());
    		dataoutputstream.writeBytes(dataString[i]);
    	}
    }
    
    @Override
    public void processPacket(NetHandler nethandler)
    {
    	// If player exists in the map, get it. Otherwise reflect it and put it in the map.
    	EntityPlayerMP entityPlayerMp = null;
    	if(playerMap.containsKey(nethandler))
    	{
    		entityPlayerMp = playerMap.get(nethandler);
    	}
    	else if(nethandler instanceof NetServerHandler)
    	{
    		try
    		{
    			entityPlayerMp = (EntityPlayerMP)ModLoader.getPrivateValue(NetServerHandler.class, nethandler, 4);
    		}
    		catch (NoSuchFieldException nosuchfieldexception)
    		{
    			System.out.println("Error getting player from NetServerHandler.");
    			nosuchfieldexception.printStackTrace();
    		}
    	}
    	
        ModLoaderMp.HandleAllPackets(this, entityPlayerMp);
    }
    
    @Override
    public int getPacketSize()
    {
    	// modId
        int packetSize = 1;
        
        // packetType
        packetSize++;
        
        // dataInt
        packetSize++;
    	packetSize += dataInt.length * Integer.SIZE;
    	
    	// dataFloat
    	packetSize++;
        packetSize += dataFloat.length * Float.SIZE;
        
        // dataString
        packetSize++;
        for(int i = 0; i < dataString.length; i++)
        {
        	packetSize++;
        	packetSize += dataString[i].length();
        }
        
        return packetSize;
    }
    
    public int modId;
    public int packetType;
    public int[] dataInt = new int[0];
    public float[] dataFloat = new float[0];
    public String[] dataString = new String[0];
    
    private static Map<NetHandler, EntityPlayerMP> playerMap = new HashMap<NetHandler, EntityPlayerMP>();
}
