package net.minecraft.src;

import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.minecraft.server.MinecraftServer;

// SDK
@SuppressWarnings({ "rawtypes", "unchecked" })
public class mod_SdkFps extends SdkBaseModMultiplayer
{
	// Blocks
    public static final Block blockPlayerSpawner = (new SdkBlockPlayerSpawner(117, ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png"))).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockPlayerSpawner");
    public static final Block blockWeaponSpawnerGuns = (new SdkBlockWeaponSpawnerGuns(118, ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png"))).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockWeaponSpawnerGuns");
    public static final Block blockWeaponSpawnerNotGuns = (new SdkBlockWeaponSpawnerNotGuns(119, ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png"))).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockWeaponSpawnerNotGuns");
    
    // Items
    public static final Item itemPlayerSpawner = (new SdkItemUltimateTool(2528, blockPlayerSpawner)).setIconIndex(ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png")).setItemName("itemPlayerSpawner");
    public static final Item itemWeaponSpawnerGuns = (new SdkItemUltimateTool(2529, blockWeaponSpawnerGuns)).setIconIndex(ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png")).setItemName("itemWeaponSpawnerGuns");
    public static final Item itemWeaponSpawnerNotGuns = (new SdkItemUltimateTool(2533, blockWeaponSpawnerNotGuns)).setIconIndex(ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png")).setItemName("itemWeaponSpawnerNotGuns");
    
    // For multiple textures
    public static final int blockSpawnerSideIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerSide.png");
    public static final int blockWeaponGunsSpawnerGunGatlingIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGunGatling.png");
    public static final int blockWeaponNotGunsSpawnerGrenadeIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenade.png");
    public static final int blockWeaponNotGunsSpawnerGrenadeStunIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeStun.png");
    public static final int blockWeaponNotGunsSpawnerGrenadeSmokeIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeSmoke.png");
    public static final int blockWeaponNotGunsSpawnerParachuteIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerParachute.png");
    public static final int blockWeaponNotGunsSpawnerGrapplingHookIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrapplingHook.png");
    public static final int blockWeaponNotGunsSpawnerGrenadeStickyIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeSticky.png");
    public static final int blockWeaponNotGunsSpawnerNightvisionIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerNightvision.png");
    public static final int blockWeaponNotGunsSpawnerScubaTankIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerScubaTank.png");
    
    // Properties
    public static boolean blocksMineable = true;
    public static boolean blocksExplodable = true;
    public static boolean blocksBurnable = true;
    public static boolean enablePlayerSpawners = true;
    public static boolean useRandomPlayerSpawner = true;
    public static boolean enableWeaponSpawners = true;
    public static boolean enableSpawnerRecipes = false;
    public static boolean restrictInventory = false;
    public static boolean respawnWithPistol = false;
    public static int weaponSpawnerRespawnDelay = 30;
    public static boolean playersDropInventory = true;
    
    // Spawn points
    private static ArrayList<SdkPoint3d<Integer>> spawnPoints = new ArrayList<SdkPoint3d<Integer>>();
    
    // Weapon maps
    public static Map<Byte, SdkItemGun> weaponGunsIdItemMap = new HashMap<Byte, SdkItemGun>();
    public static Map<Integer, Integer> weaponGunsIdTextureMap = new HashMap<Integer, Integer>();
    
    public static Map<Byte, Item> weaponNotGunsIdItemMap = new HashMap<Byte, Item>();
    public static Map<Integer, Integer> weaponNotGunsIdTextureMap = new HashMap<Integer, Integer>();
    
    private static Set<Integer> allowedItemIds = new HashSet<Integer>();
    
    // For unbreakable blocks
	private static boolean areBlocksMineable = true;
	private static float[] blockHardness;
	private static boolean areBlocksExplodable = true;
	private static float[] blockResistance;
	private static boolean areBlocksBurnable = true;
	private static int[] blockChanceToEncourageFire;
	private static int[] blockAbilityToCatchFire;
    
    static
    {
    	loadProperties();
    }
    
    private static void loadProperties()
    {
    	Properties properties = new Properties();
    	try
    	{
    		String propertiesPath = mod_SdkFps.class.getProtectionDomain().getCodeSource().getLocation().toURI().getPath();
    		propertiesPath = propertiesPath.substring(0, propertiesPath.lastIndexOf('/') + 1);
    		propertiesPath += "mod_SdkFps.properties";
    		
    		properties.load(new FileInputStream(propertiesPath));
    		String property;
    		
    		property = properties.getProperty("blocksMineable");
    		blocksMineable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("blocksExplodable");
    		blocksExplodable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("blocksBurnable");
    		blocksBurnable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enablePlayerSpawners");
    		enablePlayerSpawners = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("useRandomPlayerSpawner");
    		useRandomPlayerSpawner = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enableWeaponSpawners");
    		enableWeaponSpawners = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enableSpawnerRecipes");
    		enableSpawnerRecipes = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("restrictInventory");
    		restrictInventory = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("respawnWithPistol");
    		respawnWithPistol = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("weaponSpawnerRespawnDelay");
    		weaponSpawnerRespawnDelay = Integer.parseInt(property.trim());
    		
    		property = properties.getProperty("playersDropInventory");
    		playersDropInventory = Boolean.parseBoolean(property.trim());
        }
        catch(Exception e)
        {
        	System.out.println("Error loading properties.");
			e.printStackTrace();
        }
    }
	
	public mod_SdkFps()
	{
		super();
		
		ModLoader.RegisterBlock(blockPlayerSpawner);
		ModLoader.RegisterBlock(blockWeaponSpawnerGuns);
		ModLoader.RegisterBlock(blockWeaponSpawnerNotGuns);
		
		ModLoader.RegisterTileEntity(SdkTileEntityPlayerSpawner.class, "Player Spawner");
		ModLoader.RegisterTileEntity(SdkTileEntityWeaponSpawnerGuns.class, "Weapon Spawner Guns");
		ModLoader.RegisterTileEntity(SdkTileEntityWeaponSpawnerNotGuns.class, "Weapon Spawner Not Guns");
	}
	
	@Override
	public String Version()
	{
		return "1.3_01v4";
	}

	@Override
	public void SetRecipes(CraftingManager craftingmanager) {
		SdkTools.setShapedRecipe(craftingmanager, enableSpawnerRecipes, new ItemStack(itemPlayerSpawner, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Item.porkCooked
        });
		SdkTools.setShapedRecipe(craftingmanager, enableSpawnerRecipes, new ItemStack(itemWeaponSpawnerGuns, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Item.gunpowder
        });
		SdkTools.setShapedRecipe(craftingmanager, enableSpawnerRecipes, new ItemStack(itemWeaponSpawnerNotGuns, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Block.ladder
        });
	}
	
	@Override
    public void ModsLoaded()
    {
    	buildWeaponMaps();
    }

	public static void buildWeaponMaps()
	{
		// Add guns to map
		weaponGunsIdItemMap.put((byte)0, (SdkItemGun)mod_SdkGuns.itemGunAk47);
    	weaponGunsIdItemMap.put((byte)1, (SdkItemGun)mod_SdkGuns.itemGunMp5);
    	weaponGunsIdItemMap.put((byte)2, (SdkItemGun)mod_SdkGuns.itemGunShotgun);
    	weaponGunsIdItemMap.put((byte)3, (SdkItemGun)mod_SdkGuns.itemGunDeagle);
    	weaponGunsIdItemMap.put((byte)4, (SdkItemGun)mod_SdkGuns.itemGunRocketLauncher);
    	weaponGunsIdItemMap.put((byte)5, (SdkItemGun)mod_SdkGuns.itemGunRocketLauncherLaser);
    	weaponGunsIdItemMap.put((byte)6, (SdkItemGun)mod_SdkGuns.itemGunSniper);
    	weaponGunsIdItemMap.put((byte)7, (SdkItemGun)mod_SdkGuns.itemGunFlamethrower);
    	weaponGunsIdItemMap.put((byte)8, (SdkItemGun)mod_SdkGuns.itemGunSg552);
    	weaponGunsIdItemMap.put((byte)9, (SdkItemGun)mod_SdkGuns.itemGunGatling);
    	
    	// Add guns and ammo to allowed items
    	for(byte i = 0; i < weaponGunsIdItemMap.size(); i++)
    	{
    		SdkItemGun weaponGun = weaponGunsIdItemMap.get(i);
    		allowedItemIds.add(weaponGun.shiftedIndex);
			allowedItemIds.add(weaponGun.requiredBullet.shiftedIndex);
    	}
    	
    	// Add gun textures
    	weaponGunsIdTextureMap.put(0, mod_SdkGuns.sentryTextureIndices[0]);
    	weaponGunsIdTextureMap.put(1, mod_SdkGuns.sentryTextureIndices[1]);
    	weaponGunsIdTextureMap.put(2, mod_SdkGuns.sentryTextureIndices[2]);
    	weaponGunsIdTextureMap.put(3, mod_SdkGuns.sentryTextureIndices[3]);
    	weaponGunsIdTextureMap.put(4, mod_SdkGuns.sentryTextureIndices[4]);
    	weaponGunsIdTextureMap.put(5, mod_SdkGuns.sentryTextureIndices[5]);
    	weaponGunsIdTextureMap.put(6, mod_SdkGuns.sentryTextureIndices[6]);
    	weaponGunsIdTextureMap.put(7, mod_SdkGuns.sentryTextureIndices[7]);
    	weaponGunsIdTextureMap.put(8, mod_SdkGuns.sentryTextureIndices[8]);
    	weaponGunsIdTextureMap.put(9, blockWeaponGunsSpawnerGunGatlingIndexInTexture);
    	
    	// Add not guns to map
    	weaponNotGunsIdItemMap.put((byte)0, Item.itemsList[mod_SdkGuns.blockSentry.blockID]);
    	weaponNotGunsIdItemMap.put((byte)1, mod_SdkGuns.itemGrenade);
    	weaponNotGunsIdItemMap.put((byte)2, mod_SdkGuns.itemGrenadeSmoke);
    	weaponNotGunsIdItemMap.put((byte)3, mod_SdkGuns.itemGrenadeStun);
    	weaponNotGunsIdItemMap.put((byte)4, mod_SdkGuns.itemGrenadeSticky);
    	weaponNotGunsIdItemMap.put((byte)5, mod_SdkUtility.itemParachute);
    	weaponNotGunsIdItemMap.put((byte)6, mod_SdkUtility.itemNightvisionGoggles);
    	weaponNotGunsIdItemMap.put((byte)7, mod_SdkUtility.itemScubaTank);
    	weaponNotGunsIdItemMap.put((byte)8, mod_SdkGrapplingHook.itemGrapplingHook);
    	
    	// Add not guns to allowed items
    	for(byte i = 0; i < weaponNotGunsIdItemMap.size(); i++)
    		allowedItemIds.add(weaponNotGunsIdItemMap.get(i).shiftedIndex);
    	
    	// Add not gun textures
    	weaponNotGunsIdTextureMap.put(0, mod_SdkGuns.blockSentry.blockIndexInTexture);
    	weaponNotGunsIdTextureMap.put(1, blockWeaponNotGunsSpawnerGrenadeIndexInTexture);
    	weaponNotGunsIdTextureMap.put(2, blockWeaponNotGunsSpawnerGrenadeSmokeIndexInTexture);
    	weaponNotGunsIdTextureMap.put(3, blockWeaponNotGunsSpawnerGrenadeStunIndexInTexture);
    	weaponNotGunsIdTextureMap.put(4, blockWeaponNotGunsSpawnerGrenadeStickyIndexInTexture);
    	weaponNotGunsIdTextureMap.put(5, blockWeaponNotGunsSpawnerParachuteIndexInTexture);
    	weaponNotGunsIdTextureMap.put(6, blockWeaponNotGunsSpawnerNightvisionIndexInTexture);
    	weaponNotGunsIdTextureMap.put(7, blockWeaponNotGunsSpawnerScubaTankIndexInTexture);
    	weaponNotGunsIdTextureMap.put(8, blockWeaponNotGunsSpawnerGrapplingHookIndexInTexture);

    	// Add spawners to allowed items
    	allowedItemIds.add(itemPlayerSpawner.shiftedIndex);
    	allowedItemIds.add(itemWeaponSpawnerGuns.shiftedIndex);
    	allowedItemIds.add(itemWeaponSpawnerNotGuns.shiftedIndex);
	}
	
	@Override
	public void OnTickInGameTick(MinecraftServer minecraftServer)
	{
		// Set mineable
		if(blocksMineable != areBlocksMineable)
			setBlocksMineable(blocksMineable);

		// Set explodable
		if(blocksExplodable != areBlocksExplodable)
			setBlocksExplodable(blocksExplodable);

		// Set burnable
		if(blocksBurnable != areBlocksBurnable)
			setBlocksBurnable(areBlocksBurnable);
		
		// Restrict inventory
		if(restrictInventory && minecraftServer.worldMngr.playerEntities.size() > 0)
		{
			for(EntityPlayerMP e : new ArrayList<EntityPlayerMP>(minecraftServer.configManager.playerEntities))
			{
				InventoryPlayer inventory = e.inventory;
				
				Set<Integer> itemIds = new HashSet<Integer>();
				for(int i = 0; i < inventory.mainInventory.length; i++)
				{
					if(inventory.mainInventory[i] != null)
					{
						if(!allowedItemIds.contains(inventory.mainInventory[i].getItem().shiftedIndex))
						{
							inventory.mainInventory[i] = null;
						}
						else if(itemIds.contains(inventory.mainInventory[i].itemID))
						{
							if(inventory.mainInventory[i].getItem() instanceof SdkItemGun)
							{
								SdkItemGun itemGun = (SdkItemGun)inventory.mainInventory[i].getItem();
								inventory.mainInventory[i] = new ItemStack(itemGun.requiredBullet, itemGun.requiredBullet.maxStackSize);
							}
						}
						else
						{
							itemIds.add(inventory.mainInventory[i].itemID);
						}
					}
				}
			}
		}
	}
	
	@Override
	public void HandleLogin(EntityPlayerMP entityPlayerMp)
	{
		sendProperties(entityPlayerMp);
		setSpawnPoint(entityPlayerMp, false);
	}

	private void sendProperties(EntityPlayerMP entityPlayerMp)
	{
		int[] dataInt = new int[10];
		dataInt[0] = blocksMineable ? 1 : 0;
		dataInt[1] = blocksExplodable ? 1 : 0;
		dataInt[2] = blocksBurnable ? 1 : 0;
		dataInt[3] = enablePlayerSpawners ? 1 : 0;
        dataInt[4] = useRandomPlayerSpawner ? 1: 0;
		dataInt[5] = enableWeaponSpawners ? 1 : 0;
		dataInt[6] = enableSpawnerRecipes ? 1 : 0;
		dataInt[7] = restrictInventory ? 1 : 0;
		dataInt[8] = respawnWithPistol ? 1 : 0;
		dataInt[9] = weaponSpawnerRespawnDelay;
		
		Packet200ModLoader packet200ModLoader = new Packet200ModLoader();
		packet200ModLoader.packetType = 0;
		packet200ModLoader.dataInt = dataInt;
		ModLoaderMp.SendPacketTo(this, entityPlayerMp, packet200ModLoader);
	}

	private void setBlocksMineable(boolean setMineable)
	{
		try
		{
			if(!setMineable)
			{
				// Make unmineable
				blockHardness = new float[Block.blocksList.length];
				
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						blockHardness[i] = Block.blocksList[i].blockHardness;
						
						if(i != Block.glass.blockID || !mod_SdkGuns.bulletsDestroyGlass)
							Block.blocksList[i].blockHardness = 6000000F;
					}
				}
			}
			else
			{
				// Make mineable
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						Block.blocksList[i].blockHardness = blockHardness[i];
					}
				}
			}
			
			areBlocksMineable = setMineable;
		}
        catch(Exception e)
        {
        	System.out.println("Error setting blocks mineable: " + setMineable + ".");
			e.printStackTrace();
        }
	}
	
	private void setBlocksExplodable(boolean setExplodable)
	{
		try
		{
			if(!setExplodable)
			{
				// Make unexplodable
				blockResistance = new float[Block.blocksList.length];
				
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						blockResistance[i] = Block.blocksList[i].blockResistance;
						
						if(i != Block.glass.blockID || !mod_SdkGuns.bulletsDestroyGlass)
							Block.blocksList[i].blockResistance = 6000000F;
					}
				}
			}
			else
			{
				// Make explodable
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						Block.blocksList[i].blockResistance = blockResistance[i];
					}
				}
			}
			
			areBlocksExplodable = setExplodable;
		}
        catch(Exception e)
        {
        	System.out.println("Error setting blocks explodable: " + setExplodable + ".");
			e.printStackTrace();
        }
	}

	private void setBlocksBurnable(boolean setBurnable)
	{
		try
		{
			if(!setBurnable)
			{
				// Make unburnable
				blockChanceToEncourageFire = ((int[])ModLoader.getPrivateValue(BlockFire.class, Block.fire, 0)).clone();
				blockAbilityToCatchFire = ((int[])ModLoader.getPrivateValue(BlockFire.class, Block.fire, 1)).clone();
				
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 0, new int[blockChanceToEncourageFire.length]);
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 1, new int[blockAbilityToCatchFire.length]);
				
				// Leave some things burnable
		        SdkTools.setBurnRate(Block.tnt.blockID, 15, 100);
				mod_SdkGuns.setBurnRates();
			}
			else
			{
				// Make burnable
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 0, blockChanceToEncourageFire);
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 1, blockAbilityToCatchFire);
			}
			
			areBlocksBurnable = setBurnable;
		}
        catch(Exception e)
        {
        	System.out.println("Error setting blocks burnable: " + setBurnable + ".");
			e.printStackTrace();
        }
	}

	public static void registerPlayerSpawner(int i, int j, int k)
	{
		synchronized(mod_SdkFps.class)
		{
			SdkPoint3d<Integer> spawnPoint = new SdkPoint3d<Integer>(i, j, k);
			
			if(!spawnPoints.contains(spawnPoint))
			{
				spawnPoints.add(spawnPoint);
			}
		}
	}

	public static void unregisterPlayerSpawner(int i, int j, int k)
	{
		synchronized(mod_SdkFps.class)
		{
			SdkPoint3d<Integer> spawnPoint = new SdkPoint3d<Integer>(i, j, k);
			
			if(spawnPoints.contains(spawnPoint))
			{
				spawnPoints.remove(new SdkPoint3d<Integer>(i, j, k));
			}
		}
	}
	
	public static void setSpawnPoint(EntityPlayer entityPlayer, boolean respawn)
	{
		setSpawnPoint(entityPlayer, entityPlayer.posX, entityPlayer.posY, entityPlayer.posZ, respawn);
	}
	
	public static void setSpawnPoint(EntityPlayer entityPlayer, double prevX, double prevY, double prevZ, boolean respawn)
	{
		synchronized(mod_SdkFps.class)
		{
			if(enablePlayerSpawners && spawnPoints.size() > 0)
			{
				SdkPoint3d<Integer> spawnPoint;
				if(useRandomPlayerSpawner)
					spawnPoint = spawnPoints.get(SdkTools.random.nextInt(spawnPoints.size()));
				else
				{
					double minDistanceSquared = Double.MAX_VALUE;
					int minIndex = 0;
					for(int i = 0; i < spawnPoints.size(); i++)
					{
						SdkPoint3d<Integer> tempSpawnPoint = spawnPoints.get(i);
						
						double distanceX = tempSpawnPoint.x - prevX;
						double distanceY = tempSpawnPoint.y - prevY;
						double distanceZ = tempSpawnPoint.z - prevZ;
						
						double distanceSquared = distanceX * distanceX + distanceY * distanceY + distanceZ * distanceZ;
						if(distanceSquared < minDistanceSquared)
						{
							minDistanceSquared = distanceSquared;
							minIndex = i;
						}
					}
					
					spawnPoint = spawnPoints.get(minIndex);
				}
				
				entityPlayer.setPosition(spawnPoint.x + 0.5D, spawnPoint.y + 0.0625F, spawnPoint.z + 0.5D);
			}
			
			if((respawn || mod_SdkGuns.playerInventoryEmpty(entityPlayer.inventory)) && respawnWithPistol)
			{
				setPlayerInventory(entityPlayer);
			}
		}
	}

	private static void setPlayerInventory(EntityPlayer entityPlayer)
	{
		setPlayerInventoryItem(entityPlayer, 0, new ItemStack(mod_SdkGuns.itemGunDeagle));
		setPlayerInventoryItem(entityPlayer, 9, new ItemStack(mod_SdkGuns.itemBulletMedium, mod_SdkGuns.itemBulletMedium.maxStackSize));
		setPlayerInventoryItem(entityPlayer, 10, new ItemStack(mod_SdkGuns.itemBulletMedium, mod_SdkGuns.itemBulletMedium.maxStackSize));
	}
	
	private static boolean setPlayerInventoryItem(EntityPlayer entityPlayer, int index, ItemStack itemStack)
	{
		if(entityPlayer.inventory.mainInventory[index] == null)
		{
			entityPlayer.inventory.mainInventory[index] = itemStack;
			return true;
		}
		else
		{
			return entityPlayer.inventory.addItemStackToInventory(itemStack);
		}
	}

	@Override
	public void OnMainMenu() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void InitBlocks() {
		// TODO Auto-generated method stub
		
	}
}
