package net.minecraft.src;

import java.util.ArrayList;
import java.util.Map;

import net.minecraft.server.MinecraftServer;

// SDK
@SuppressWarnings({ "rawtypes", "unchecked" })
public class mod_SdkUtility extends SdkBaseModMultiplayer
{
	// Blocks
    public static final Block blockGrinder = (new SdkBlockGrinder(120, ModLoader.addOverride("/terrain.png", "/sdk/blockNuke.png"))).setHardness(3.5F).setStepSound(Block.soundStoneFootstep).setBlockName("blockGrinder");
    
    // Items
    public static final Item itemGoldCoin = (new Item(2509)).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/itemGoldCoin.png")).setItemName("itemGoldCoin");
    public static final Item itemLightometer = (new Item(2510)).setIconIndex(ModLoader.getUniqueSpriteIndex("/gui/items.png")).setItemName("itemLightometer");
    public static final Item itemNightvisionGoggles = (new ItemArmor(2513, -1, 0, 0)).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/itemNightvisionGoggles.png")).setItemName("itemNightvision");
    public static final Item itemScubaTank = (new ItemArmor(2521, -1, 0, 1)).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/itemScubaTank.png")).setItemName("itemScubaTank");
    public static final Item itemParachute = (new SdkItemParachute(2523, 0)).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/itemParachute.png")).setItemName("itemParachute");
    public static final Item itemTelescope = (new SdkItemTelescope(2534)).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/itemTelescope.png")).setItemName("itemTelescope");
	
	public mod_SdkUtility()
    {
		super();
		
		ModLoader.RegisterBlock(blockGrinder);
		
		ModLoader.RegisterEntityID(SdkEntityParachute.class, "Parachute", ModLoader.getUniqueEntityId());
		
		ModLoader.RegisterTileEntity(SdkTileEntityGrinder.class, "Grinder");
    }
	
	@Override
	public String Version()
	{
		return "1.3_01v4";
	}
	
	@Override
	public void OnTickInGameTick(MinecraftServer minecraftServer)
	{
		for(EntityPlayerMP e : new ArrayList<EntityPlayerMP>(minecraftServer.configManager.playerEntities))
		{
			ItemStack itemstack = e.inventory.armorInventory[2];
	        if(itemstack != null && itemstack.itemID == itemScubaTank.shiftedIndex)
	        {
	        	e.air = e.maxAir;
	        }
		}
	}

	@Override
	public void SetRecipes(CraftingManager craftingmanager) {
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemGoldCoin, 4), new Object[] {
            "#", Character.valueOf('#'), Item.ingotGold
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemLightometer, 1), new Object[] {
        	" # ", "#X#", " # ", Character.valueOf('#'), Item.ingotIron, Character.valueOf('X'), Item.lightStoneDust
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemNightvisionGoggles, 1), new Object[] {
        	"#X#", Character.valueOf('#'), Item.diamond, Character.valueOf('X'), Item.silk
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemScubaTank, 1), new Object[] {
        	"###", "# #", "###", Character.valueOf('#'), Item.ingotIron
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemParachute, 1), new Object[] {
        	"###", "XYX", "YYY", Character.valueOf('#'), Block.cloth, Character.valueOf('X'), Item.silk, Character.valueOf('Y'), Item.leather
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(blockGrinder, 1), new Object[] {
        	"###", "#X#", "###", Character.valueOf('#'), Block.cobblestone, Character.valueOf('X'), Item.diamond
        });
		SdkTools.setShapedRecipe(craftingmanager, true, new ItemStack(itemTelescope), new Object[] {
        	"#", "X", "X", Character.valueOf('#'), Item.diamond, Character.valueOf('X'), Item.ingotGold
        });
	}
	
	@Override
	public void AddEntityTrackers(Map<Class, Pair<Integer, Integer>> map)
	{
		map.put(SdkEntityParachute.class, new Pair<Integer, Integer>(160, 3));
	}
	
	@Override
	public void AddEntityTrackerEntries(Map<Class, Integer> map)
	{
		map.put(SdkEntityParachute.class, 120);
	}
	
	@Override
	public void HandlePacket(Packet200ModLoader packet200ModLoader, EntityPlayerMP entityPlayerMp)
	{
		switch(packet200ModLoader.packetType)
		{
			case 0:
			{
				handleParachuteKey(entityPlayerMp);
				break;
			}
		}
	}
	
	public void handleParachuteKey(EntityPlayer entityPlayer)
	{
		ItemStack itemstack = entityPlayer.inventory.armorInventory[2];
		if(itemstack != null && itemstack.itemID == itemParachute.shiftedIndex)
			useParachute(itemstack, SdkTools.minecraftserver.worldMngr, entityPlayer);
	}
	
	private void useParachute(ItemStack itemstack, World world, EntityPlayer entityplayer)
	{
		if(!SdkEntityParachute.onGroundOrInWater(world, entityplayer))
    	{
	        itemstack.damageItem(1);
            if(itemstack.stackSize == 0)
            {
            	boolean removed = false;
            	for(int i = 0; i < entityplayer.inventory.armorInventory.length; i++)
            	{
            		if(entityplayer.inventory.armorInventory[i] == itemstack)
            		{
            			entityplayer.inventory.armorInventory[i] = null;
            			removed = true;
            			break;
            		}
            	}
            	
            	if(!removed)
            	{
	            	for(int i = 0; i < entityplayer.inventory.mainInventory.length; i++)
	            	{
	            		if(entityplayer.inventory.mainInventory[i] == itemstack)
	            		{
	            			entityplayer.inventory.mainInventory[i] = null;
	            			removed = true;
	            			break;
	            		}
	            	}
            	}
            }
	        
	        world.playSoundAtEntity(entityplayer, "sdk.parachute", 0.5F, 1.0F / (SdkTools.random.nextFloat() * 0.1F + 0.95F));
	        
	        if(!world.multiplayerWorld)
	        {
	        	world.entityJoinedWorld(new SdkEntityParachute(world, entityplayer));
	        }
    	}
	}
    
    public static void useZoom() { }

	@Override
	public void OnMainMenu() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void InitBlocks() {
		// TODO Auto-generated method stub
		
	}
}
